﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace RaycasterX
{
	public class Camera
	{
		public Vector2 Position;
		public Vector2 Direction;
		public Vector2 Plane;
		public float Radius = .3f;
		public float MoveSpeed = 3f;
		public float RotationSpeed = 3f;

        public void Update(float dt, GamePadState gps, Map map)
		{
			HandleTranslation(gps, map, dt);
			HandleRotation(gps, dt);
		}

        private void HandleTranslation(GamePadState gps, Map map, float dt)
		{
			//allow touchpad movement...
            Vector2 move = Direction * gps.ThumbSticks.Left.Y * (dt * MoveSpeed);

			//or dpad movement
			if (gps.DPad.Up == ButtonState.Pressed)
				move = Direction * (dt * MoveSpeed);
            if (gps.DPad.Down == ButtonState.Pressed)
				move = -Direction * (dt * MoveSpeed);

            //Change by Patrick Pelletier - 20091120
            UpdatePosition(move, map);
            /*
			if (move.X != 0)
			{
				//take into account the camera's radius when moving
				float rX = (move.X > 0) ? Radius : -Radius;

				//make sure there is no wall where we're trying to move
				if (map.WorldMap[(int)(Position.X + move.X + rX), (int)Position.Y] == 0)
					Position.X += move.X;
			}
			if (move.Y != 0)
			{
				//take into account the camera's radius when moving
				float rY = (move.Y > 0) ? Radius : -Radius;

				//make sure there is no wall where we're trying to move
				if (map.WorldMap[(int)Position.X, (int)(Position.Y + move.Y + rY)] == 0)
					Position.Y += move.Y;
			}
            */
            //End of change
		}

        //Addition by Patrick Pelletier - 20091120
        private void UpdatePosition(Vector2 move, Map map)
        {
            if (move.X != 0)
            {
                //take into account the camera's radius when moving
                float rX = (move.X > 0) ? Radius : -Radius;

                //make sure there is no wall where we're trying to move
                if (map.WorldMap[(int)(Position.X + move.X + rX), (int)Position.Y] == 0)
                    Position.X += move.X;
            }
            if (move.Y != 0)
            {
                //take into account the camera's radius when moving
                float rY = (move.Y > 0) ? Radius : -Radius;

                //make sure there is no wall where we're trying to move
                if (map.WorldMap[(int)Position.X, (int)(Position.Y + move.Y + rY)] == 0)
                    Position.Y += move.Y;
            }
        }

        private void HandleRotation(GamePadState gps, float dt)
		{
			//allow touchpad rotation...
            float rot = -gps.ThumbSticks.Left.X * (dt * RotationSpeed);

			//or DPad rotation
            if (gps.DPad.Right == ButtonState.Pressed)
				rot = -(dt * RotationSpeed);
            if (gps.DPad.Left == ButtonState.Pressed)
				rot = (dt * RotationSpeed);

            //Change by Patrick Pelletier - 20091120
            UpdateRotation(rot);
            /*
			//calculate the sin and cos values
			float sRot = (float)Math.Sin(rot);
			float cRot = (float)Math.Cos(rot);

			//rotate the Direction vector
			float odX = Direction.X;
			Direction.X = Direction.X * cRot - Direction.Y * sRot;
			Direction.Y = odX * sRot + Direction.Y * cRot;

			//rotate the Plane vector
			float opX = Plane.X;
			Plane.X = Plane.X * cRot - Plane.Y * sRot;
			Plane.Y = opX * sRot + Plane.Y * cRot;
            */
            //End of change
		}

        //Addition by Patrick Pelletier - 20091120
        private void UpdateRotation(float rot)
        {
            //calculate the sin and cos values
            float sRot = (float)Math.Sin(rot);
            float cRot = (float)Math.Cos(rot);

            //rotate the Direction vector
            float odX = Direction.X;
            Direction.X = Direction.X * cRot - Direction.Y * sRot;
            Direction.Y = odX * sRot + Direction.Y * cRot;

            //rotate the Plane vector
            float opX = Plane.X;
            Plane.X = Plane.X * cRot - Plane.Y * sRot;
            Plane.Y = opX * sRot + Plane.Y * cRot;
        }

        //Addition by Patrick Pelletier - 20091120
        public void Update(float dt, KeyboardState kbs, Map map)
        {
            HandleTranslation(kbs, map, dt);
            HandleRotation(kbs, dt);
        }

        //Addition by Patrick Pelletier - 20091120
        private void HandleTranslation(KeyboardState kbs, Map map, float dt)
        {
            Vector2 move = Vector2.Zero;

            if (kbs.IsKeyDown(Keys.Up))
                move = Direction * (dt * MoveSpeed);
            if (kbs.IsKeyDown(Keys.Down))
                move = -Direction * (dt * MoveSpeed);

            UpdatePosition(move, map);
        }

        //Addition by Patrick Pelletier - 20091120
        private void HandleRotation(KeyboardState kbs, float dt)
        {
            float rot = 0;

            if (kbs.IsKeyDown(Keys.Right))
                rot = -(dt * RotationSpeed);
            if (kbs.IsKeyDown(Keys.Left))
                rot = (dt * RotationSpeed);

            UpdateRotation(rot);
        }
	}
}
